@@ -29,6 +29,7 @@
 #include "sourceoutput.h"
 #include "server.h"
 #include "streamrestore.h"
+#include "module.h"
 
 #include <QMetaEnum>
 
@@ -39,6 +40,13 @@ AbstractModel::AbstractModel(const MapBaseQObject *map, QObject *parent)
     : QAbstractListModel(parent)
     , m_map(map)
 {
+    Context::instance()->ref();
+    //deref context after we've deleted this object
+    //see https://bugs.kde.org/show_bug.cgi?id=371215
+    connect(this, &QObject::destroyed, []() {
+        Context::instance()->unref();
+    });
+
     connect(m_map, &MapBaseQObject::added, this, &AbstractModel::onDataAdded);
     connect(m_map, &MapBaseQObject::removed, this, &AbstractModel::onDataRemoved);
 }
@@ -90,6 +98,11 @@ int AbstractModel::role(const QByteArray &roleName) const
     return m_roles.key(roleName, -1);
 }
 
+Context *AbstractModel::context() const
+{
+    return Context::instance();
+}
+
 void AbstractModel::initRoleNames(const QMetaObject &qobjectMetaObject)
 {
     m_roles[PulseObjectRole] = QByteArrayLiteral("PulseObject");
@@ -347,4 +360,10 @@ StreamRestoreModel::StreamRestoreModel(QObject *parent)
     initRoleNames(StreamRestore::staticMetaObject);
 }
 
+ModuleModel::ModuleModel(QObject *parent)
+    : AbstractModel(&context()->modules(), parent)
+{
+    initRoleNames(Module::staticMetaObject);
+}
+
 } // QPulseAudio
